/***********************************************************************
 * GBA hardware interrupt handler code
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 **********************************************************************/

#ifndef	_GBA_INTERRUPT_H
#define	_GBA_INTERRUPT_H

#include "types.h"


// interface for all interrupt routines
typedef void (*InterruptFunction) (void);


// hardware interrupt registers
#define pREG_INTERRUPT_MASTER_ENABLE	(volatile u16 *)0x4000208	// master interrupt enable switch
#define REG_INTERRUPT_MASTER_ENABLE		*pREG_INTERRUPT_MASTER_ENABLE

#define pREG_INTERRUPT_ENABLE			(volatile u16 *)0x4000200	// register to set which interrupts are active
#define REG_INTERRUPT_ENABLE			*pREG_INTERRUPT_ENABLE

#define pREG_INTERRUPT_STATUS  			(volatile u16 *)0x4000004	// interrupt status register
#define REG_INTERRUPT_STATUS			*pREG_INTERRUPT_STATUS

#define	pREG_INTERRUPT_PROCESSED        (volatile u16 *)0x4000202	// interrupt processed register (used for telling the hardware the interrupt has been processed)
#define REG_INTERRUPT_PROCESSED			*pREG_INTERRUPT_PROCESSED

// enable bits for the master interrupt register
#define	INTERRUPT_MASTER_ENABLE	0x0001	// turn on interrupts in the GBA (REG_INTERRUPT_MASTER_ENABLE)

// NOTE: some interrupts (vblank, hblank, vcount, must be enables in both the
//       REG_INTERRUPT_ENABLE and REG_INTERRUPT_STATUS registers

// enable bits for the various GBA hardware interrupts (set in REG_INTERRUPT_ENABLE)
#define INTERRUPT_VBLANK    0x0001	// vertical blank interval interrupt
#define INTERRUPT_HBLANK	0x0002	// horizontal blank interval interrupt
#define INTERRUPT_VCOUNT	0x0004	// scan line count interrupt
#define INTERRUPT_TIMER0	0x0008	// timer 0 interrupt
#define INTERRUPT_TIMER1	0x0010	// timer 1 interrupt
#define INTERRUPT_TIMER2	0x0020	// timer 2 interrupt
#define INTERRUPT_TIMER3	0x0040	// timer 3 interrupt
#define INTERRUPT_SERIAL	0x0080	// serial communications port interrupt
#define INTERRUPT_DMA0		0x0100	// DMA 0 interrupt
#define INTERRUPT_DMA1		0x0200	// DMA 1 interrupt
#define INTERRUPT_DMA2		0x0400	// DMA 2 interrupt
#define INTERRUPT_DMA3		0x0800	// DMA 3 interrupt
#define INTERRUPT_KEYBOARD	0x1000	// keyboard interrupt
#define INTERRUPT_CART		0x2000	// cartridge interrupt

// bits for the interrupt status register (REG_INTERRUPT_STATUS)
#define	INTERRUPT_STATUS_VBLANK_ON		0x0001	// set if a vblank is occuring
#define	INTERRUPT_STATUS_HBLANK_ON		0x0002	// set if an hblank is occuring
#define	INTERRUPT_STATUS_VCOUNT			0x0004	// set if the vertical count register (REG_VCOUNT) is equal to the value set in the vcount bits of this register
#define	INTERRUPT_STATUS_VBLANK_ENABLE	0x0008	// enables the vblank interrupt
#define	INTERRUPT_STATUS_HBLANK_ENABLE	0x0010	// enables the hblank interrupt
#define	INTERRUPT_STATUS_VCOUNT_ENABLE	0x0020	// enables the vcount interrupt
#define	INTERRUPT_STATUS_VCOUNT_MASK	0xFFC0	// set these bits to the scanline to do a vcount interrupt on

#endif

