/******************************************************************************
 * powerup object code
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *****************************************************************************/

#include "powerup.h"
#include "sprite.h"
#include "game.h"
#include "world.h"

s24f8_vect2D powerupSize = {int_to_s24f8(4), int_to_s24f8(4)};


/********************************************************************
 * launch a powerup
 *
 * IN:		world position of the powerup
 * 			amount of health the powerup gives if picked up
 *
 * RETURNS:	ptr to the powerup object
 *******************************************************************/
PowerupStruct *Powerup_New (s24f8_vect2D *pPos, u16 health)
{
u32 objID;
PowerupStruct *pPowerup;
ANGLE tmpAngle;
s24f8 speed;

	// grab a new object
	objID = Object_New(OBJECT_POWERUP);
	if (objID == OBJECT_ID_NULL)
	{	// error... no unused objects left
		return(NULL);
	}

	// get the pointer to it
	pPowerup = (PowerupStruct *)Object_getPtrS(objID);

	// init it
	pPowerup->pos = *pPos;

	pPowerup->health = health;

	// get a random speed and direction for the powerup
	tmpAngle = Random_Get32() & 0x0FF;
	speed = Random_Get32() % int_to_s24f8(1);
	pPowerup->speed.x = s24f8_mul(COS_TABLE[tmpAngle], speed);
	pPowerup->speed.y = s24f8_mul(SIN_TABLE[tmpAngle], speed);

	return(pPowerup);
}


/**********************************************************************
 * update a powerup object
 *********************************************************************/
u32 Powerup_Update (ObjectStruct *pObj)
{
PowerupStruct *pPowerup = (PowerupStruct *)pObj;

	if (pPowerup->health == 0)
	{	// this powerup got picked up
		return(1);
	}

	// move the bullet
	pPowerup->pos.x += s24f8_mul(pPowerup->speed.x, frameTimeF);
	pPowerup->pos.y += s24f8_mul(pPowerup->speed.y, frameTimeF);

	if (Sprite_OnScreenUpdate(&pPowerup->pos, &powerupSize, &gWorldPos,
							  SPRITE_COLOR_MODE_256, SPRITE_SIZE_8, HEART_TILENUM<<1) == 0)
	{	// sprite is off-screen... kill it!
		return(1);
	}

	return(0);
}

